module CRC5D5_test_bench;
		parameter integer initial_value=-1;
		parameter integer poly_width=5;
		parameter integer data_width=poly_width;
		parameter integer maximum_no_of_errors=3;
		parameter integer data_length=10;
	
		reg clock=0;
		reg sync_reset=1;
		reg hold=0, receiver_hold=0;
		wire [poly_width-1:0] crc_sender,crc_receiver;
		reg done=0;
		integer i,test_loops,error_counter=0;
	
		reg error=0;
		reg [data_width-1:0] data=0;
		wire  [data_width-1:0] receive_data=data^ error;
	

	always #10 clock=~clock;

	initial begin
			#5;
			test_loops=0;
//Test 1 Make sure no crc error occurs
			repeat (100) begin
				test_loops=test_loops+1;
				if (test_loops%100==0) $display("Test Loops=%d",test_loops);
				
				sync_reset=1; 
				@(negedge clock);
				@(negedge clock);
				sync_reset=0; hold=0;
				for (i=0;i<data_length;i=i+1) begin
					data=$random;
					@(negedge clock);
				end
				hold=1;
				data=crc_sender;
				@(negedge clock);
				done=1;
				if (crc_receiver !==0) $display("Miss Detectio or Programming Error");
				@(negedge clock);
				done=0;
			end
//Make sure crc error occurs
			test_loops=0;
			repeat (100) begin
				test_loops=test_loops+1;
				if (test_loops%1000==0) $display("Test Loops=%d",test_loops);
			
				sync_reset=1; 
				@(negedge clock);
				@(negedge clock);
				sync_reset=0; hold=0;error_counter=0;
				for (i=0;i<data_length;i=i+1) begin
					data=$random;
					if (i==0) begin
						error=1;//We add at least one error. 
						error_counter=error_counter+1;
					end else begin//Limit maxinum no of errors
						if (error_counter <maximum_no_of_errors) begin
							error=$random;//$random error addition
							error_counter=error_counter+1;
						end
					end
					@(negedge clock);
				end
				hold=1;//Stop CRC generation in sender
				data=crc_sender;//CRC polynominal are sent
				@(negedge clock);
				done=1;
				if (crc_receiver ==0) $display("Miss Error Detection or Programming Error Test Loops=%d",test_loops);
				@(negedge clock);
				done=0;
			end



			$finish;
	end


CRC5_D5_Module #(.initial_value(initial_value),.poly_width(poly_width),.data_width(data_width)) crc_sender( clock,
						    data,sync_reset,hold,
							   crc_sender);

CRC5_D5_Module #(.initial_value(initial_value),.poly_width(poly_width),.data_width(data_width)) crc_receiver( clock,
						    receive_data,sync_reset,receiver_hold,
						   crc_receiver);

endmodule
