///////////////////////////////////////////////////////////////////////
// File:  CRC5_D2.v                             
// Date:  Tue May  3 09:00:37 2005                                                      
//                                                                     
// Copyright (C) 1999-2003 Easics NV.                 
// This source file may be used and distributed without restriction    
// provided that this copyright statement is not removed from the file 
// and that any derivative work contains the original copyright notice
// and the associated disclaimer.
//
// THIS SOURCE FILE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS
// OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// Purpose: Verilog module containing a synthesizable CRC function
//   * polynomial: (0 2 5)
//   * data width: 2
//                                                                     
// Info: tools@easics.be
//       http://www.easics.com                                  
///////////////////////////////////////////////////////////////////////


module CRC5_D2;

  // polynomial: (0 2 5)
  // data width: 2
  // convention: the first serial data bit is D[1]
  function [4:0] nextCRC5_D2;

    input [1:0] Data;
    input [4:0] CRC;

    reg [1:0] D;
    reg [4:0] C;
    reg [4:0] NewCRC;

  begin

    D = Data;
    C = CRC;

    NewCRC[0] = D[0] ^ C[3];
    NewCRC[1] = D[1] ^ C[4];
    NewCRC[2] = D[0] ^ C[0] ^ C[3];
    NewCRC[3] = D[1] ^ C[1] ^ C[4];
    NewCRC[4] = C[2];

    nextCRC5_D2 = NewCRC;

  end

  endfunction

endmodule

