///////////////////////////////////////////////////////////////////////
// File:  CRC5_D4.v                             
// Date:  Tue May  3 07:13:44 2005                                                      
//                                                                     
// Copyright (C) 1999-2003 Easics NV.                 
// This source file may be used and distributed without restriction    
// provided that this copyright statement is not removed from the file 
// and that any derivative work contains the original copyright notice
// and the associated disclaimer.
//
// THIS SOURCE FILE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS
// OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// Purpose: Verilog module containing a synthesizable CRC function
//   * polynomial: (0 2 5)
//   * data width: 4
//                                                                     
// Info: tools@easics.be
//       http://www.easics.com                                  
///////////////////////////////////////////////////////////////////////


module CRC5_D4;

  // polynomial: (0 2 5)
  // data width: 4
  // convention: the first serial data bit is D[3]
  function [4:0] nextCRC5_D4;

    input [3:0] Data;
    input [4:0] CRC;

    reg [3:0] D;
    reg [4:0] C;
    reg [4:0] NewCRC;

  begin

    D = Data;
    C = CRC;

    NewCRC[0] = D[3] ^ D[0] ^ C[1] ^ C[4];
    NewCRC[1] = D[1] ^ C[2];
    NewCRC[2] = D[3] ^ D[2] ^ D[0] ^ C[1] ^ C[3] ^ C[4];
    NewCRC[3] = D[3] ^ D[1] ^ C[2] ^ C[4];
    NewCRC[4] = D[2] ^ C[0] ^ C[3];

    nextCRC5_D4 = NewCRC;

  end

  endfunction

endmodule

