///////////////////////////////////////////////////////////////////////
// File:  CRC5_D5.v                             
// Date:  Tue May  3 09:53:42 2005                                                      
//                                                                     
// Copyright (C) 1999-2003 Easics NV.                 
// This source file may be used and distributed without restriction    
// provided that this copyright statement is not removed from the file 
// and that any derivative work contains the original copyright notice
// and the associated disclaimer.
//
// THIS SOURCE FILE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS
// OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// Purpose: Verilog module containing a synthesizable CRC function
//   * polynomial: (0 2 5)
//   * data width: 5
//                                                                     
// Info: tools@easics.be
//       http://www.easics.com                                  
///////////////////////////////////////////////////////////////////////

//Add interface for test May.4.2005 Tak.Sugawara

module  CRC5_D5_Module #(parameter integer initial_value=-1, poly_width=5,data_width=5) (input clock,
					     input [data_width-1:0] data,
					     input sync_reset,hold,
					     output reg [poly_width-1:0] crc);

	always @(posedge clock) begin
	  if (sync_reset) crc <=initial_value;
		else if (hold) crc <=crc;
		else crc<=nextCRC5_D5(data,crc);
	end

  // polynomial: (0 2 5)
  // data width: 5
  // convention: the first serial data bit is D[4]
  function [4:0] nextCRC5_D5;

    input [4:0] Data;
    input [4:0] CRC;

    reg [4:0] D;
    reg [4:0] C;
    reg [4:0] NewCRC;

  begin

    D = Data;
    C = CRC;

    NewCRC[0] = D[3] ^ D[0] ^ C[0] ^ C[3];
    NewCRC[1] = D[4] ^ D[1] ^ C[1] ^ C[4];
    NewCRC[2] = D[3] ^ D[2] ^ D[0] ^ C[0] ^ C[2] ^ C[3];
    NewCRC[3] = D[4] ^ D[3] ^ D[1] ^ C[1] ^ C[3] ^ C[4];
    NewCRC[4] = D[4] ^ D[2] ^ C[2] ^ C[4];

    nextCRC5_D5 = NewCRC;

  end

  endfunction

endmodule

