///////////////////////////////////////////////////////////////////////
// File:  CRC8_D4.v                             
// Date:  Wed May  4 09:56:21 2005                                                      
//                                                                     
// Copyright (C) 1999-2003 Easics NV.                 
// This source file may be used and distributed without restriction    
// provided that this copyright statement is not removed from the file 
// and that any derivative work contains the original copyright notice
// and the associated disclaimer.
//
// THIS SOURCE FILE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS
// OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// Purpose: Verilog module containing a synthesizable CRC function
//   * polynomial: (0 1 2 8)
//   * data width: 4
//                                                                     
// Info: tools@easics.be
//       http://www.easics.com                                  
///////////////////////////////////////////////////////////////////////



//Add interface for test May.4.2005 Tak.Sugawara

module  CRC8_D4_Module #(parameter integer initial_value=-1, poly_width=8,data_width=4) (input clock,
					     input [data_width-1:0] data,
					     input sync_reset,hold,
					     output reg [poly_width-1:0] crc);

	always @(posedge clock) begin
	  if (sync_reset) crc <=initial_value;
		else if (hold) crc <=crc;
		else crc<=nextCRC8_D4(data,crc);
	end
  // polynomial: (0 1 2 8)
  // data width: 4
  // convention: the first serial data bit is D[3]
  function [7:0] nextCRC8_D4;

    input [3:0] Data;
    input [7:0] CRC;

    reg [3:0] D;
    reg [7:0] C;
    reg [7:0] NewCRC;

  begin

    D = Data;
    C = CRC;

    NewCRC[0] = D[0] ^ C[4];
    NewCRC[1] = D[1] ^ D[0] ^ C[4] ^ C[5];
    NewCRC[2] = D[2] ^ D[1] ^ D[0] ^ C[4] ^ C[5] ^ C[6];
    NewCRC[3] = D[3] ^ D[2] ^ D[1] ^ C[5] ^ C[6] ^ C[7];
    NewCRC[4] = D[3] ^ D[2] ^ C[0] ^ C[6] ^ C[7];
    NewCRC[5] = D[3] ^ C[1] ^ C[7];
    NewCRC[6] = C[2];
    NewCRC[7] = C[3];

    nextCRC8_D4 = NewCRC;

  end

  endfunction

endmodule

